package de.dassit.dassadmin;

import java.io.IOException;
import java.util.Hashtable;
import java.util.logging.Logger;

import javax.naming.CommunicationException;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.StartTlsRequest;
import javax.naming.ldap.StartTlsResponse;
import javax.net.ssl.SSLSession;

public class ActiveDirectoryTool {
	private static final Logger log = Logger.getLogger(ActiveDirectoryTool.class
			.getName());

	protected LdapContext ctx;
	protected SSLSession ssl;
	protected StartTlsResponse tls;

	protected String serverURL;

	protected String bindDn;

	protected String bindPassword;

	protected String container;

	protected boolean ignoreMissingUsers;

	protected String errorMessage = "Es ist ein Fehler beim Schreiben des Passworts in Active Directory aufgetreten. Bitte wenden Sie sich an den Support.";

	private synchronized void connect() throws NamingException, IOException {
		if (ctx != null)
			return;

		Hashtable<String, String> env = new Hashtable<String, String>(11);
		env.put(Context.INITIAL_CONTEXT_FACTORY,
				"com.sun.jndi.ldap.LdapCtxFactory");
		env.put(Context.PROVIDER_URL, serverURL);
		env.put(Context.SECURITY_AUTHENTICATION, "simple");
		env.put(Context.SECURITY_PRINCIPAL, bindDn);
		env.put(Context.SECURITY_CREDENTIALS, bindPassword);

		ctx = new InitialLdapContext(env, null);

		// Start TLS
		StartTlsResponse tls = (StartTlsResponse) ctx
				.extendedOperation(new StartTlsRequest());
		ssl = tls.negotiate();

	}

	private synchronized void disconnect() throws NamingException, IOException {
		if (ctx != null) {
			if (tls != null) {
				tls.close();
				tls = null;
				ssl = null;
			}
			ctx.close();
			ctx = null;
		}
	}

	private String translateUsername(String username) throws NamingException,
			IOException {

		SearchControls sc = new SearchControls();
		sc.setSearchScope(SearchControls.SUBTREE_SCOPE);

		NamingEnumeration<SearchResult> ne = null;

		// log.info(relativeDn+baseDn+"/"+filter);
		try {
			ne = ctx.search(container, "samAccountName=" + username, sc);
		} catch (CommunicationException cex) {
			log.warning("AD connection failed (" + cex.getMessage()
					+ ")");
			disconnect();
		}
		while (ne.hasMore()) {
			SearchResult sr = (SearchResult) ne.next();
			// log.info(sr.getNameInNamespace());

			return sr.getNameInNamespace();
		}

		return null;
	}

	public void setPassword(String username, String cleartext)
			throws NamingException {
		try {
			disconnect();
			connect();

			String dn = translateUsername(username);

			log.info(dn);
			if (dn == null) {
				String message = "user " + username + " not found.";

				if (ignoreMissingUsers) {
					log.info(message);
					return;
				} else {
					throw new NamingException(message);
				}
			}

			String quoted = "\"" + cleartext + "\"";
			char unicodePwd[] = quoted.toCharArray();
			byte pwdArray[] = new byte[unicodePwd.length * 2];
			for (int i = 0; i < unicodePwd.length; i++) {
				pwdArray[i * 2 + 1] = (byte) (unicodePwd[i] >>> 8);
				pwdArray[i * 2 + 0] = (byte) (unicodePwd[i] & 0xff);
			}

			ModificationItem[] mods = new ModificationItem[1];
			mods[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
					new BasicAttribute("UnicodePwd", pwdArray));
			ctx.modifyAttributes(dn, mods);

			disconnect();
		} catch (Exception ex) {
			ex.printStackTrace();
			throw new NamingException(errorMessage);
		}
	}

	public String getBindDn() {
		return bindDn;
	}

	public void setBindDn(String bindDn) {
		this.bindDn = bindDn;
	}

	public String getBindPassword() {
		return bindPassword;
	}

	public void setBindPassword(String bindPassword) {
		this.bindPassword = bindPassword;
	}

	public String getContainer() {
		return container;
	}

	public void setContainer(String container) {
		this.container = container;
	}

	public String getServerURL() {
		return serverURL;
	}

	public void setServerURL(String serverURL) {
		this.serverURL = serverURL;
	}

	public boolean getIgnoreMissingUsers() {
		return ignoreMissingUsers;
	}

	public void setIgnoreMissingUsers(boolean ignoreMissingUsers) {
		this.ignoreMissingUsers = ignoreMissingUsers;
	}

	public String getErrorMessage() {
		return errorMessage;
	}

	public void setErrorMessage(String errorMessage) {
		this.errorMessage = errorMessage;
	}
}
