package de.dassit.dassadmin;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.naming.directory.SearchResult;

import de.dassit.ldap.Connection;
import de.dassit.ldap.Entry;
import de.dassit.ldap.EntryList;

public class DassComputer extends SmbUser {
	private static final Logger log = Logger.getLogger(DassComputer.class
			.getName());

	static String extraObjectClasses;
	static String templateContainer = "ou=dassadmin";

	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<DassComputer>() {
			public void addInstance(SearchResult sr) {
				entries.add(new DassComputer(sr));
			}
		};
	}

	public String getTemplateContainer() {
		return templateContainer;
	}

	public void setTemplateContainer(String templateContainer) {
		DassComputer.templateContainer = templateContainer;
	}

	public DassComputer() {
		super();
	}

	public DassComputer(SearchResult sr) {
		super(sr);
	}

	public DassComputer(String uid, int uidNumber, int gidNumber, String cn,
			String sn, String home) {
		super(uid, uidNumber, gidNumber, cn, sn, home);
	}

	@Override
	protected void setNamingParameters() {
		setNamingAttribute(defaultNamingAttribute);
		setObjectClasses(new String[] { "top", "account", "ipHost",
				"posixAccount", "sambaSamAccount" });
		addObjectClassesCSV(extraObjectClasses);
		setContainer(defaultContainer);
	}

	public void setExtraObjectClasses(String csv) {
		extraObjectClasses = csv;
	}

	@Override
	public void setName(String s) {
		super.setName(s);
		put("uid", s + "$");
	}

	@Override
	public String getName() {
		String n = get("uid");
		return n.substring(0, n.length() - 1);
	}

	@Override
	public void save() throws NamingException {
		if (get("ipHostNumber").equals("auto")) {
			Network factory = new Network();
			Network net = (Network) factory.fetch(getNetworkName());
			put("ipHostNumber", net.getNextIpAddress());
		}
		super.save();
	}

	public List<String> getAliases() {
		final String name = getName();
		List<String> result = new ArrayList<String>();

		for (String s : getAttributeMultiValue(namingAttribute)) {
			if (!s.equals(name))
				result.add(s);
		}

		return result;
	}

	public void removeAliases() {
		final List<String> aliases = getAliases();
		for (String a : aliases) {
			removeAliasName(a);
		}
	}

	public void addAliasName(String alias) {
		appendAttributeValue(namingAttribute, alias);
	}

	public void removeAliasName(String alias) {
		removeAttributeValue(namingAttribute, alias);
	}

	@Override
	public void rename(String newName) throws NamingException {
		put("uid", newName + "$");
		save();
		super.rename(newName);
	}

	public static synchronized String getNextName() throws NamingException {
		List<DassComputer> result = DassComputer.searchComputers(
				getDefaultNamingAttribute(DassComputer.class.getName())
						+ "=templateComputer", templateContainer);
		DassComputer template = result.get(0);

		String nextName = template.get("host");

		int i = rindexOfFirstDigit(nextName);
		int num = 0;
		String prefix = nextName.substring(0, i);
		String digits = nextName.substring(i, nextName.length());
		if (!digits.isEmpty())
			num = Integer.parseInt(digits);

		log.info("next name " + nextName);
		DassComputer factory = new DassComputer();
		while (factory.search(
				getDefaultNamingAttribute(DassComputer.class.getName()) + "="
						+ nextName,
				getDefaultContainer(DassComputer.class.getName())).size() != 0) {
			num++;
			nextName = prefix + num;
			log.info("trying name " + nextName);
		}
		log.info(nextName + " ok");
		template.put("host", prefix + (num + 1));
		template.save();
		return nextName;
	}

	private static int rindexOfFirstDigit(String s) {
		int i = s.length() - 1;
		while (i >= 0 && Character.isDigit(s.charAt(i))) {
			i--;
		}
		return i + 1;
	}

	public void addDeviceReference(String name) {
		IpDevice factory = new IpDevice();
		IpDevice device = (IpDevice) factory.fetch(name);
		changeSeeAlso(null, device.getDN());
	}

	public void removeDeviceReference(String name) {
		IpDevice factory = new IpDevice();
		IpDevice device = (IpDevice) factory.fetch(name);
		changeSeeAlso(device.getDN(), null);
	}

	public List<String> getDeviceReferences() {
		ArrayList<String> list = new ArrayList<String>();
		int numberOfValues = getAttributeSize("seeAlso");

		for (int i = 0; i < numberOfValues; i++) {
			String v = getAttribute("seeAlso", i);
			if (isDeviceDn(v)) {
				list.add(getNameFromDn(v));
			}
		}
		return list;
	}

	public void setNetworkName(String n) {
		String old = getNetworkDn();

		Network factory = new Network();
		Network network = (Network) factory.fetch(n);
		String newName = network.getDN();
		changeSeeAlso(old, newName);
	}

	public String getNameFromDn(String dn) {
		if (dn == null)
			return null;

		String rdn = dn.split(",")[0];
		String name = rdn.split("=")[1];

		return name;
	}

	public String getNetworkName() {
		return getNameFromDn(getNetworkDn());
	}

	public String getProfileName() {
		return getNameFromDn(getProfileDn());
	}

	public void setProfileName(String n) {
		String old = getProfileDn();

		DassProfile factory = new DassProfile();
		DassProfile profile = (DassProfile) factory.fetch(n);
		String newName = profile.getDN();
		changeSeeAlso(old, newName);
	}

	public String getNetworkDn() {
		if (getAttributeSize("seeAlso") == 0)
			return null;

		String networksContainer = getDefaultContainer(Network.class.getName());

		for (String s : getAttributeMultiValue("seeAlso")) {
			if (s.indexOf(networksContainer) != -1)
				return s;
		}
		return null;
	}

	public boolean isDeviceDn(String dn) {
		if (getAttributeSize("seeAlso") == 0)
			return false;

		String devicesContainer = getDefaultContainer(IpDevice.class.getName());
		for (String s : getAttributeMultiValue("seeAlso")) {
			if (s.equals(dn) && s.indexOf(devicesContainer) != -1)
				return true;
		}
		return false;
	}

	public String getProfileDn() {
		if (getAttributeSize("seeAlso") == 0)
			return null;

		String profilesContainer = getDefaultContainer(DassProfile.class
				.getName());

		for (String s : getAttributeMultiValue("seeAlso")) {
			if (s.indexOf(profilesContainer) != -1)
				return s;
		}
		return null;
	}

	public void changeSeeAlso(String oldDn, String newDn) {
		if (oldDn != null && !oldDn.isEmpty())
			removeAttributeValue("seeAlso", oldDn);
		if (newDn != null && !newDn.isEmpty())
			appendAttributeValue("seeAlso", newDn);
	}

	public static DassComputer searchByIpAddress(String address) {
		List<DassComputer> result = searchComputers("ipHostNumber=" + address);
		if (result.size() < 1)
			return null;
		return (DassComputer) result.get(0);
	}

	public static List<DassComputer> searchComputers(String filter) {
		return searchComputers(filter,
				getDefaultContainer(DassComputer.class.getName()));
	}

	public static List<DassComputer> searchComputers(String filter,
			String container) {

		EntryList<DassComputer> entryList = new EntryList<DassComputer>() {
			public void addInstance(SearchResult sr) {
				entries.add(new DassComputer(sr));
			}
		};

		Connection.getDefaultConnection().search(filter, container, entryList);

		return entryList.getEntries();
	}

	public List<DassComputer> getAll() {
		return DassComputer.searchComputers("objectClass=ipHost",
				defaultContainer);
	}

	public static List<DassComputer> getAllInNetwork(Network network) {
		return DassComputer.searchComputers("(&(objectClass=ipHost)(seeAlso="
				+ network.getDN() + "))",
				getDefaultContainer(DassComputer.class.getName()));
	}

	public static DassComputer getTemplateComputer() {
		final String className = DassComputer.class.getName();
		DassComputer c = searchComputers(
				getDefaultNamingAttribute(className) + "=templateComputer",
				templateContainer).get(0);
		c.setContainer(getDefaultContainer(className));
		c.saveAsNew();
		return c;
	}

	public static synchronized int getNextUid() throws NamingException {
		final String className = DassComputer.class.getName();

		List<DassComputer> result = searchComputers(
				getDefaultNamingAttribute(className) + "=templateComputer",
				templateContainer);
		DassComputer template = result.get(0);

		int nextUid = template.getUid();

		log.info("next uidNumber " + nextUid);
		DassComputer factory = new DassComputer();
		while (factory.search("uidNumber=" + nextUid,
				getDefaultContainer(className)).size() != 0) {
			nextUid++;
			log.info("trying uidNumber " + nextUid);
		}
		log.info(nextUid + " ok");
		template.setUid(nextUid + 1);
		template.save();
		return nextUid;
	}
}
