package de.dassit.dassadmin;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;
import java.util.regex.Pattern;

import javax.naming.directory.SearchResult;

import de.dassit.ldap.Entry;
import de.dassit.ldap.EntryList;

public class Network extends Entry {
	static String extraObjectClasses;

	static String ipStart = "127.0.0.1";
	static String ipExcludeRegexp = null;

	private static final Logger log = Logger.getLogger(Network.class.getName());

	public Network() {
		super();
	}

	public Network(SearchResult sr) {
		super(sr);
	}

	public String getIpStart() {
		return ipStart;
	}

	public void setIpStart(String ipStart) {
		Network.ipStart = ipStart;
	}

	public String getIpExcludeRegexp() {
		return ipExcludeRegexp;
	}

	public void setIpExcludeRegexp(String ipExcludeRegexp) {
		Network.ipExcludeRegexp = ipExcludeRegexp;
	}

	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<Network>() {
			public void addInstance(SearchResult sr) {
				entries.add(new Network(sr));
			}
		};
	}

	@Override
	protected void setNamingParameters() {
		setNamingAttribute(defaultNamingAttribute);
		setObjectClasses(new String[] { "top", "ipNetwork" });
		addObjectClassesCSV(extraObjectClasses);
		setContainer(defaultContainer);
	}

	public void setExtraObjectClasses(String csv) {
		extraObjectClasses = csv;
	}

	public List<? extends Entry> getAll() {
		return search("objectClass=ipNetwork", defaultContainer);
	}

	public List<String> getAllNames() {
		List<? extends Entry> all = getAll();
		List<String> result = new ArrayList<String>(all.size());

		for (Entry e : all) {
			Network n = (Network) e;
			result.add(n.getName());
		}
		return result;
	}

	public boolean containsAddress(String address) throws UnknownHostException {
		int netmask = addressToInt(get("ipNetmaskNumber"));
		int netaddr = addressToInt(get("ipNetworkNumber"));
		int host = addressToInt(address);

		return (host & netmask) == netaddr;
	}

	public static String hex(int n) {
		return String.format("0x%04X", n);
	}

	protected int addressToInt(String address) throws UnknownHostException {
		Inet4Address a = (Inet4Address) InetAddress.getByName(address);
		byte[] b = a.getAddress();
		int i = ((b[0] & 0xFF) << 24) | ((b[1] & 0xFF) << 16)
				| ((b[2] & 0xFF) << 8) | ((b[3] & 0xFF) << 0);
		return i;
	}

	public String getNextIpAddress() throws NumberFormatException {
		byte[] maskBytes;
		byte[] netAddrBytes;
		byte[] addressBytes = parseIpAddress(ipStart);
		byte[] broadcastBytes = new byte[4];
		Pattern regex = null;

		if (ipExcludeRegexp != null) {
			regex = Pattern.compile(ipExcludeRegexp);
			log.info(ipExcludeRegexp);
		}

		synchronized (Network.class) {
			printIpAddressBytes(addressBytes);
			List<DassComputer> comps = DassComputer.getAllInNetwork(this);

			netAddrBytes = parseIpAddress(get("ipNetworkNumber"));
			maskBytes = parseIpAddress(get("ipNetmaskNumber"));
			for (int i = 0; i < 4; i++) {
				broadcastBytes[i] = (byte) (netAddrBytes[i] | (-1 & ~maskBytes[i]));

				addressBytes[i] = (byte) (netAddrBytes[i] | (addressBytes[i] & ~maskBytes[i]));
			}

			printIpAddressBytes(netAddrBytes);
			printIpAddressBytes(maskBytes);
			printIpAddressBytes(broadcastBytes);

			do {
				String addr = ipAddressToString(addressBytes);
				Iterator<DassComputer> iter = comps.iterator();
				boolean collision = false;

				if (regex != null && regex.matcher(addr).matches()) {
					collision = true;
					log.info(addr + " matches exclusion regexp, ignored");
				}
				while (!collision && iter.hasNext()) {
					DassComputer c = iter.next();
					if (c.get("ipHostNumber").equals(addr)) {
						collision = true;
					}
				}

				if (!collision) {
					if (IpDevice.searchByIpAddress(addr) != null)
						collision = true;
				}
				
				if(!collision){
					if(DassComputer.searchByIpAddress(addr) != null)
						collision = true;
				}

				if (!collision)
					return addr;

				incAddressBytes(addressBytes);

			} while (!Arrays.equals(addressBytes, broadcastBytes));
		}
		return "127.0.0.1";
	}

	public static void printIpAddressBytes(byte[] addressBytes) {
		log.info(ipAddressToString(addressBytes));
	}

	public static void incAddressBytes(byte[] addressBytes) {
		int carryin = 1;
		int carryout = 0;

		for (int i = 3; i >= 0; i--) {
			byte tmp = addressBytes[i];
			if (carryin != 0) {
				tmp = (byte) (tmp + 1);
				if (tmp == 0) {
					carryout = 1;
				}
			}
			addressBytes[i] = tmp;
			carryin = carryout;
			carryout = 0;
		}
	}

	public static byte[] parseIpAddress(String s) {
		String[] octets = s.split("\\.");
		byte[] addressBytes = new byte[4];

		for (int i = 0; i < 4; i++) {
			int tmp = Integer.parseInt(octets[i]);
			if (tmp < 0 || tmp > 255)
				throw new NumberFormatException("invalid IP address");
			addressBytes[i] = (byte) tmp;
		}
		return addressBytes;
	}

	public static String ipAddressToString(byte[] addressBytes) {
		StringBuffer s = new StringBuffer();
		for (int i = 0; i < 4; i++) {
			int tmp = addressBytes[i];
			if (tmp < 0)
				tmp = 256 + tmp;
			s.append(tmp);
			if (i < 3)
				s.append('.');
		}
		return s.toString();
	}
}